local encounters = HudMap:GetModule("Encounters")
local HudMap = _G.HudMap
local UnitName, UnitIsDead = _G.UnitName, _G.UnitIsDead
local parent = HudMap
local L = LibStub("AceLocale-3.0"):GetLocale("HudMap")

local marrowgar = {
	name = L["Marrowgar"],
	options = {
		spikeRadius = {
			name = L["Bone Spike Radius"],
			type = "range",
			min = 1,
			max = 5,
			step = 1,
			bigStep = 1
		},
		enableSpikes = {
			name = L["Enable Spike Display"],
			type = "toggle"
		},
		spikeColor = {
			type = "color",
			hasAlpha = true,
			name = L["Spike Color"]
		}
	},
	defaults = {
		spikeRadius = 2,
		enableSpikes = true,
		spikeColor = {r = 1, g = 0, b = 0, a = 0.6}
	},
	spikes = {},
	spikesOn = {},
	registerOn = {"SPELL_SUMMON", "UNIT_DIED"},
	-- Bone Spikes
	SPELL_SUMMON = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if self:Option("enableSpikes") then
			if spellID == 69062 or spellID == 72669 or spellID == 72670 then
				local r, g, b, a = self:Option("spikeColor")
				local radius = self:Option("spikeRadius")
				self.spikes[destGUID] = HudMap:PlaceRangeMarkerOnPartyMember("highlight", destName, radius, 8, r, g, b, a):Rotate(360, 2)
				self.spikesOn[sourceGUID] = destGUID
			end
		end
	end,
	UNIT_DIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		local marker = self.spikes[destGUID]
		if not marker then 
			local pguid = self.spikesOn[sourceGUID]
			if pguid then
				marker = self.spikes[pguid]
			end
		end
		if marker then marker:Free() end
	end
}

local festergut = {
	name = L["Festergut"],
	options = {
		enableSpores = {
			name = L["Enable Spore Display"],
			type = "toggle"
		},
		sporeColor = {
			type = "color",
			hasAlpha = true,
			name = L["Spore Color"]
		}
	},
	defaults = {
		enableSpores = true,
		sporeColor = {r = 1, g = 0.5, b = 0, a = 0.8}
	},
	registerOn = {"SPELL_AURA_APPLIED"},
	-- Gas Spore, 8yd radius
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69279 and self:Option("enableSpores") then
			local r, g, b, a = self:Option("sporeColor")
			HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 8, 12, r, g, b, a):Rotate(360, 8):Appear()
		end
	end
}

local rotface = {
	name = L["Rotface"],
	options = {
		enableSplash = {
			name = L["Enable Splash Zones"],
			type = "toggle",
			order = 110			
		},
		splashColor = {
			type = "color",
			hasAlpha = true,
			name = L["Splash Color"],
			order = 111
		},
		enableInfection = {
			name = L["Enable Infection Arrow"],
			type = "toggle",
			order = 120
		},
		infectionColor = {
			type = "color",
			hasAlpha = true,
			name = L["Infection Arrow Color"],
			order = 121
		},
		enableVile = {
			name = L["Enable Vile Gas"],
			type = "toggle",
			order = 130
		},
		vileColor = {
			type = "color",
			hasAlpha = true,
			name = L["Vile Gas Color"],
			order = 131
		}		
	},
	defaults = {
		enableSplash = true,
		splashColor = {r = 0, g = 1, b = 0, a = 0.8},
		enableInfection = true,
		infectionColor = {r = 0, g = 1, b = 0, a = 0.8},
		enableVile = true,
		vileColor = {r = 1, g = 0.5, b = 0, a = 0.8}
	},
	
	registerOn = {"SPELL_CAST_START", "SPELL_AURA_APPLIED", "SPELL_AURA_REMOVED", "UNIT_DIED", "SPELL_CAST_SUCCESS"},
	showSplashes = function()
		for index, unit in encounters.group() do
			if not UnitIsDead(unit) then
				local x, y = HudMap:GetUnitPosition(unit)
				local r, g, b, a = self:Option("splashColor")
				HudMap:PlaceRangeMarker("timer", x, y, 6, 10, r, g, b, a):Rotate(360, 10):Appear():RegisterForAlerts()
			end
		end
	end,
	-- Big Ooze Explosion
	SPELL_CAST_START = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69839 and self:Option("enableSplash") then
			encounters:Delay(self.showSplashes, 4)
		end
	end,
	
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Vile Gas
		if (spellID == 72272 or spellID == 72273) and self:Option("showVile") then
			local r, g, b, a = self:Option("vileColor")
			HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 8, 6, r, g, b, a):Rotate(360, 6):Appear():RegisterForAlerts()
			
		-- Mutated Infection
		-- TODO: Didn't work. Analyze logs.
		elseif (spellID == 69674 or spellID == 73022 or spellID == 71224 or spellID == 73023) and self:Option("enableInfection") then
			local target = encounters:GetMobTarget(36899)
			if target then
				local r, g, b, a = self:Option("infectionColor")
				self.navArrow = HudMap:AddEdge(r, g, b, a, "player", target)
			end
		end
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if spellID == 69674 or spellID == 73022 or spellID == 71224 or spellID == 73023 then
			if self.navArrow then self.navArrow:Free() end
		end
	end,
	
	-- If any small ooze dies, remove the nav arrow
	-- TODO: Try to build a queue of oozes to iterate through to know if the ooze that died was ours.
	UNIT_DIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if encounters.GuidToMobID[destGUID] == 36897 then
			if self.navArrow then self.navArrow:Free() end
		end
	end,
	
	-- If any small ooze casts merge, remove the nav arrow
	-- TODO: Try to build a queue of oozes to iterate through to know if the ooze that merged was ours.
	SPELL_CAST_SUCCESS = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if encounters.GuidToMobID[destGUID] == 36897 then
			if self.navArrow then self.navArrow:Free() end
		end
	end
}

local putricide = {
	name = L["Professor Putricide"],
	options = {
		enableGoo = {
			name = L["Enable Malleable Goo"],
			type = "toggle",
			order = 110			
		},
		gooColor = {
			type = "color",
			hasAlpha = true,
			name = L["Goo Color"],
			order = 111
		},
		enablePlague = {
			name = L["Enable Unbound Plague"],
			type = "toggle",
			order = 120
		},
		plagueColor = {
			type = "color",
			hasAlpha = true,
			name = L["Unbound Plague Color"],
			order = 121
		}
	},
	defaults = {
		enableGoo = true,
		gooColor = {r = 0, g = 1, b = 0, a = 0.8},
		enablePlague = true,
		plagueColor = {r = 1, g = 0.5, b = 0, a = 0.8},
	},
	
	registerOn = {"SPELL_CAST_SUCCESS", "SPELL_AURA_APPLIED", "SPELL_AURA_REMOVED", "UNIT_DIED"},
	plagueOn = {},
	gooTargetFunc = function()
		local gooTarget = encounters:GetMobTarget(36678)
		if gooTarget then
			local x, y = HudMap:GetUnitPosition(gooTarget)				
			local r, g, b, a = self:Option("gooColor")
			HudMap:PlaceRangeMarker("highlight", x, y, 7, 7, r, g, b, a):RegisterForAlerts():Appear()
		end
	end,
	SPELL_CAST_SUCCESS = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Malleable Goo. Find Putricide's target and mark their splash zone. Can we mark bounce spots as well?
		if (spellID == 72295 or spellID == 74280 or spellID == 72615 or spellID == 74281) and self:Option("enableGoo") then
			encounters:Delay(self.gooTargetFunc, 0.1)
		end
	end,
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Player picks up Unbound Plague
		if (spellID == 72855 or spellID == 72856) and self:Option("enablePlague") then
			local r, g, b, a = self:Option("plagueColor")
			self.plagueOn[destGUID] = HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 3, 60, r, g, b, a):Appear():Rotate(360, 10)
		end
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Player loses Unbound Plague
		if spellID == 72855 or spellID == 72856 then
			local marker = self.plagueOn[destGUID]
			if marker then
				marker:Free()
			end
		end
	end,
	UNIT_DIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Player with Plague dies
		local marker = self.plagueOn[destGUID]
		if marker then
			marker:Free()
		end
	end
}

local princes = {
	name = L["Blood Council"],
	options = {
		princeArrowOn = {
			name = L["Enable Prince Arrow"],
			type = "toggle",
			order = 100			
		},
		princeArrowColor = {
			type = "color",
			hasAlpha = true,
			name = L["Prince Arrow Color"],
			order = 101
		},		
	
		fireArrowOn = {
			name = L["Enable Fire Arrow"],
			type = "toggle",
			order = 110			
		},
		fireArrowColor = {
			type = "color",
			hasAlpha = true,
			name = L["Fire Arrow Color"],
			order = 111
		},
		
		showVortexes = {
			name = L["Enable Vortexes"],
			type = "toggle",
			order = 110			
		},
		vortexColor = {
			type = "color",
			hasAlpha = true,
			name = L["Vortex Color"],
			order = 111
		},
		
		showEmpoweredShock = {
			name = L["Enable Empowered Shocks"],
			type = "toggle",
			order = 110			
		},
		shockColor = {
			type = "color",
			hasAlpha = true,
			name = L["Empowered Shock Color"],
			order = 111
		},		
	},
	defaults = {
		fireArrowOn = true,
		fireArrowColor = {r = 1, g = 0.8, b = 0, a = 0.8},
		
		princeArrowOn = true,
		princeArrowColor = {r = 1, g = 0, b = 0, a = 0.8},
		
		showEmpoweredShock = true,
		shockColor = {1, 1, 1, 0.6},
		
		showVortexes = true,
		vortexColor = {1, 1, 1, 0.4},
	},
	registerOn = {"SPELL_CAST_START", "CHAT_MSG_RAID_BOSS_EMOTE", "SPELL_CAST_SUCCESS"},
	shockTargetFunc = function()
		local shockTarget = encounters:GetMobTarget(37970)
		if shockTarget then
			local x, y = HudMap:GetUnitPosition(shockTarget)			
			-- Timer is 45 sec on heroic?
			local r, g, b, a = self:Option("vortexColor")
			HudMap:PlaceRangeMarker("timer", x, y, 15, 10, r, g, b, a):Appear():Rotate(360, 10):RegisterForAlerts()
			encounters:Delay(function()
				HudMap:PlaceRangeMarker("highlight", x, y, 15, 30, r, g, b, a):Rotate(360, 4):RegisterForAlerts()
			end, 10)
		end
	end,
	SPELL_CAST_START = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- AOE shock vortex
		-- Seems to have a ~10 sec arming time, then persists for X seconds
		if spellID == 72037 and self:Option("showVortexes") then
			encounters:Delay(self.shockTargetFunc, 0.1)			
		-- Empowered shock vortex
		elseif (spellID == 72039 or spellID == 73037 or spellID == 73038 or spellID == 73039) and self:Option("showEmpoweredShock") then
			local r, g, b, a = self:Option("shockColor")
			encounters:RangeCheckAll("timer", 12, 5, r, g, b, a, true)
		end
	end,
	SPELL_CAST_SUCCESS = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		if self.fireArrow and (spellID == 72789 or spellID == 71393 or spellID == 72790 or spellID == 72791) then
			self.fireArrow = self.fireArrow:Free()
		end
	end,
	CHAT_MSG_RAID_BOSS_EMOTE = function(self, message, sender, language, channelString, target)
		if message and message:match(L["Empowered Flames speed toward (%S+)!"]) and target and self:Option("fireArrowOn") then
			local r, g, b, a = self:Option("fireArrowColor")
			local tank = encounters:GetMobTarget(37973)
			if tank then
				self.fireArrow = HudMap:AddEdge(r, g, b, a, tank, target)
			end
		end
	end
	-- TODO: On health swap, draw arrow to next prince 
	-- Alerts are not working (yet)
}

local bloodqueen = {
	name = L["Blood Queen Lana'thel"],
	options = {
		fear = {
			type = "toggle",
			name = L["Bloodbolt Whirl"]
		},
		fearColor = {
			type = "color",
			name = L["Bloodbolt Whirl Color"]
		}
	},
	defaults = {
		fear = true,
		fearColor = {r = 1, g = 0.4, b = 1, a = 0.6}
	},
	registerOn = {"SPELL_CAST_SUCCESS"},
	SPELL_CAST_SUCCESS = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Fear
		if spellID == 73070 and self:Option("fear") then
			local r, g, b, a = self:Option("fearColor")
			encounters:RangeCheckAll("timer", 6, 10, r, g, b, a, true)
		end
	end
}

local sindragosa = {
	registerOn = {"SPELL_AURA_APPLIED", "SPELL_AURA_REMOVED", "SPELL_CAST_SUCCESS"},
	timers = {},
	markBlastRadius = function()
		local x, y = encounters:GetCenterOfRaid(8)
		local m = HudMap:PlaceRangeMarker("timer", x, y, 25, 5, 0.3, 0.3, 1, 0.8):Rotate(360, 5):Appear():RegisterForAlerts()
	end,
	SPELL_AURA_APPLIED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Frost beacon
		if spellID == 70126 then
			HudMap:PlaceRangeMarkerOnPartyMember("timer", destName, 10, 7, 0.2, 0.9, 1, 1):Appear():Rotate(360, 7):RegisterForAlerts()
			
		-- Instability
		elseif spellID == 69766 and encounters:IsHeroic() then
			self.timers[destName] = HudMap:PlaceRangeMarkerOnPartyMember("highlight", destName, 20, nil, 0.7, 0.3, 1, 0.7):Appear():Rotate(360, 3):RegisterForAlerts()
		end
	end,
	SPELL_AURA_REMOVED = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Instability
		if spellID == 69766 and self.timers[destName] then
			self.timers[destName]:Free()
		end
	end,
	SPELL_CAST_SUCCESS = function(self, spellID, sourceName, destName, sourceGUID, destGUID)
		-- Icy Grip
		if spellID == 70117 then
			encounters:Delay(self.markBlastRadius, 1)
		end
	end
}

encounters:RegisterModule(L["Icecrown Citadel"], marrowgar, festergut, rotface, putricide, princes, bloodqueen, sindragosa)